from typing import Optional, Sequence, Dict, Any

import attrs
from numpy.random import RandomState

from vkit.pipeline.interface import (
    PipelineStep,
    NoneTypePipelineStepResource,
    PipelineStepFactory,
    PipelineState,
)
from vkit.engine.image import image_factory


@attrs.define
class BackgroundPipelineStepConfig:
    image_configs: Sequence[Dict[str, Any]]
    state_key_height: str = 'height'
    state_key_width: str = 'width'
    state_key_background_image: str = 'background_image'


class BackgroundPipelineStep(
    PipelineStep[BackgroundPipelineStepConfig, NoneTypePipelineStepResource]
):

    def __init__(
        self,
        config: BackgroundPipelineStepConfig,
        resource: Optional[NoneTypePipelineStepResource] = None,
    ):
        super().__init__(config, resource)

        self.image_aggregator = image_factory.create(self.config.image_configs)

    def run(self, state: PipelineState, rnd: RandomState):
        height = state.get_value(self.config.state_key_height, int)
        width = state.get_value(self.config.state_key_width, int)

        background_image = self.image_aggregator.run(
            {
                'height': height,
                'width': width,
            },
            rnd,
        )
        state.set_value(self.config.state_key_background_image, background_image)


background_step_factory = PipelineStepFactory(BackgroundPipelineStep)
