from typing import Optional, Sequence

import attrs
from numpy.random import RandomState

from vkit.utility import rnd_choice
from vkit.pipeline.interface import (
    PipelineStep,
    NoneTypePipelineStepResource,
    PipelineStepFactory,
    PipelineState,
)


@attrs.define
class PageShapePipelineStepConfig:
    aspect_ratios: Sequence[float]
    height_min: int
    height_max: int
    state_key_height: str = 'height'
    state_key_width: str = 'width'


class PageShapePipelineStep(
    PipelineStep[PageShapePipelineStepConfig, NoneTypePipelineStepResource]
):

    def __init__(
        self,
        config: PageShapePipelineStepConfig,
        resource: Optional[NoneTypePipelineStepResource] = None,
    ):
        super().__init__(config, resource)

    def run(self, state: PipelineState, rnd: RandomState):
        height = rnd.randint(self.config.height_min, self.config.height_max + 1)
        aspect_ratio = rnd_choice(rnd, self.config.aspect_ratios)
        width = round(aspect_ratio * height)
        assert height > 0 and width > 0

        state.set_value(self.config.state_key_height, height)
        state.set_value(self.config.state_key_width, width)


page_shape_step_factory = PipelineStepFactory(PageShapePipelineStep)
