from collections import defaultdict

from github import Github
from tqdm import tqdm
import iolite as io


def get_stargazers_and_save(github_token, user_repo, output_json):
    github = Github(github_token)
    repo = github.get_repo(user_repo)
    user_names = []
    for user in tqdm(repo.get_stargazers()):
        user_names.append(user.login)
    io.write_json(
        output_json,
        {
            'user_repo': user_repo,
            'user_names': user_names,
        },
        indent=2,
    )


def group_as_user_name_to_user_repos(collect_folder, ouptut_json):
    user_name_to_user_repos = defaultdict(list)
    for json_file in io.folder(collect_folder, exists=True).glob('*.json'):
        data = io.read_json(json_file)
        for user_name in data['user_names']:
            user_name_to_user_repos[user_name].append(data['user_repo'])

    repo_cnt_to_num_users = defaultdict(int)
    for user_repos in user_name_to_user_repos.values():
        repo_cnt_to_num_users[len(user_repos)] += 1

    print(sorted(repo_cnt_to_num_users.items(), reverse=True))

    io.write_json(ouptut_json, user_name_to_user_repos, indent=2)


def get_user_and_save(
    github_token,
    user_name_to_user_repos_json,
    output_folder,
    disable_skip_exists=False,
):
    github = Github(github_token)
    user_name_to_user_repos = io.read_json(user_name_to_user_repos_json)
    out_fd = io.folder(output_folder, touch=True)
    num_users_has_email = 0
    num_users = 0
    for user_name in tqdm(user_name_to_user_repos):
        output_json = out_fd / f'{user_name}.json'
        if output_json.exists() and not disable_skip_exists:
            print(f'skip {user_name}')
            continue
        user = github.get_user(user_name)
        if user.email:
            num_users_has_email += 1
        num_users += 1
        io.write_json(output_json, user.raw_data, indent=2, ensure_ascii=False)
    print(f'num_users_has_email = {num_users_has_email}')
    print(f'num_users = {num_users}')
