from numpy.random import RandomState
import iolite as io

from vkit.element import LexiconCollection
from vkit.engine.font.type import FontCollection
from vkit.engine.font import font_factory, FontEngineRunConfig
from vkit.engine.char_sampler import char_sampler_factory, CharSamplerEngineResource
from vkit.engine.char_and_font_sampler import (
    char_and_font_sampler_factory,
    CharAndFontSamplerEngineResource,
)

from vkit.element.visualization import visualize_mask, visualize_boxes


def demo():
    lexicon_collection = LexiconCollection.from_file(
        '$VKIT_LEXICON_DATA/lexicon_collection_combined/chinese.json'
    )
    print('lexicon_collection loaded.')

    font_collection = FontCollection.from_folder('$VKIT_FONT_DATA/font_collection')
    print('font_collection loaded.')

    char_sampler_aggregator = char_sampler_factory.create(
        '$VKIT_FONT_DATA/font_engine_test/char_sampler_aggregator.json',
        CharSamplerEngineResource(lexicon_collection=lexicon_collection),
    )
    print('char_sampler_aggregator loaded.')

    char_and_font_sampler = char_and_font_sampler_factory.create(
        {},
        CharAndFontSamplerEngineResource(
            lexicon_collection=lexicon_collection,
            font_collection=font_collection,
            char_sampler_aggregator=char_sampler_aggregator,
        )
    )

    font_aggregator = font_factory.create(
        '$VKIT_FONT_DATA/font_engine_test/font_engine_aggregator.json'
    )
    print('font_engine_aggregator loaded.')

    out_fd = io.folder('$VKIT_FONT_DATA/font_engine_test/debug', expandvars=True, touch=True)
    rnd = RandomState(0)
    for idx in range(20):
        print(idx)

        height = 32
        width = rnd.randint(640, 1280)

        char_and_font = char_and_font_sampler.run(
            config={
                'height': height,
                'width': width
            },
            rnd=rnd,
        )
        assert char_and_font

        result = font_aggregator.run(
            config=FontEngineRunConfig(
                height=height,
                width=width,
                chars=char_and_font.chars,
                font_variant=char_and_font.font_variant,
            ),
            rnd=rnd,
        )
        assert result

        result.image.to_file(out_fd / f'{idx}.png')
        visualize_mask(result.mask).to_file(out_fd / f'{idx}_mask.png')
        visualize_boxes(result.image, boxes=[char_box.box for char_box in result.char_boxes
                                             ]).to_file(out_fd / f'{idx}_char_boxes.png')
